''************************************
''*     SD Data Logger Object v1.0   *
''*    For use with Hitt Consulting  *
''*       SD Data Logger Module      *
''*     (C) 2006 Hitt Consulting     *
''*     terry@hittconsulting.com     *
''************************************
'************************************
'*      Thanks to Michael Green     *
'*         for starting this.       *
'************************************

' Uses "FullDuplexSerial" Object

'Init(SPin) : Sets up object. Communication is on pin "SPin"
'WakeUp  : Wakes up Data Logger from sleep mode
'Version  : Returns Data Logger firmware version
'LastError  : Returns last error code
'WriteMemory(addr, ptr, count) : Write data to Data Logger SRAM addr=SRAM address
'ReadMemory(addr, ptr, count)  : Read data from Data Logger SRAM addr=SRAM address
'Pacing(value)  : Sets the sending speed (Propeller doesn't need any delay)
'ConvertCard  : Converts media card to 32MB
'Sleep  : Puts module in low power mode
'OpenFile(fileNameStr)  : Opens a file. fileNameStr is a pointer to a string
'CloseFile  : Closes the open file
'FileSize  : Returns the open file size
'Seek(pos)  : Seeks to position "pos" within the open file
'Terminator(char)  : Set the string terminator character
'AppendStr(str)  : Appends a string to the open file. str is a pointer to a string
'ReadStr(str)  : Reads a string from the open file. str is a pointer to a string array
'AppendBlock(ptr, count)  : Appends a block of memory to the open file
'ReadBlock(ptr, count)  : Reads a block of memory from the open file


VAR
  BYTE TerminatorChar


OBJ
  ser : "FullDuplexSerial"


PUB Init(SPin)
  ser.start(SPin,SPin,%1111,19200)
  TerminatorChar:=10
  

PUB WakeUp
  ser.rxflush  
  ser.str(STRING("!WU"))               ' Wake-up module
  

PUB Version | ver1, ver2
  ser.rxflush  
  ser.str(STRING("!VR"))
  ver1 := ser.rxtime(500)            ' Get module version
  ver2 := ser.rxtime(500)
  RETURN ver1 + ver2*256


PUB LastError
  ser.rxflush  
  ser.str(STRING("!LE"))
  RETURN ser.rxtime(500)


PUB WriteMemory(addr, ptr, count)
  ser.rxflush  
  REPEAT WHILE count > 255
    ser.str(STRING("!WM"))
    ser.tx(addr // $FF)
    ser.tx(addr / $FF)
    ser.tx(255)
    REPEAT 255
      ser.tx(BYTE[ptr])
      ptr++
    count-=255
    addr+=255
  IF count > 0    
    ser.str(STRING("!WM"))
    ser.tx(addr // $FF)
    ser.tx(addr / $FF)
    ser.tx(count)
    REPEAT count
      ser.tx(BYTE[ptr])
      ptr++
  RETURN ser.rxtime(500)
      

PUB ReadMemory(addr, ptr, count)
  ser.rxflush  
  REPEAT WHILE count > 255
    ser.str(STRING("!RM"))
    ser.tx(addr // $FF)
    ser.tx(addr / $FF)
    ser.tx(255)
    REPEAT 255
      BYTE[ptr]:=ser.rxtime(500)
      ptr++
    count-=255
    addr+=255
  IF count > 0    
    ser.str(STRING("!RM"))
    ser.tx(addr // $FF)
    ser.tx(addr / $FF)
    ser.tx(count)
    REPEAT count
      BYTE[ptr]:=ser.rxtime(500)
      ptr++
  RETURN ser.rxtime(500)

  
PUB Pacing(value)
  ser.rxflush  
  ser.str(STRING("!PV"))
  ser.tx(value)
  RETURN ser.rxtime(500)


PUB ConvertCard
  ser.rxflush  
  ser.str(STRING("!CC"))
  RETURN ser.rxtime(5000)
    

PUB Sleep
  ser.rxflush  
  ser.str(STRING("!SL"))

  
PUB OpenFile(fileNameStr)
  ser.rxflush  
  ser.str(STRING("!OF"))
  ser.str(fileNameStr)
'  ser.rxflush
  RETURN ser.rxtime(5000)


PUB CloseFile
  ser.rxflush
  ser.str(STRING("!CF"))
  RETURN ser.rxtime(5000)


PUB FileSize | value
  ser.rxflush
  ser.str(STRING("!FS"))
  value.BYTE[0]:=ser.rxtime(500)
  value.BYTE[1]:=ser.rxtime(500)
  value.BYTE[2]:=ser.rxtime(500)
  value.BYTE[3]:=ser.rxtime(500)
  RETURN ser.rxtime(500)


PUB Seek(pos)
  ser.rxflush
  ser.str(STRING("!GP"))
  ser.tx(pos.BYTE[0])
  ser.tx(pos.BYTE[1])
  ser.tx(pos.BYTE[2])
  ser.tx(pos.BYTE[3])
  RETURN ser.rxtime(5000)
  
   
PUB Terminator(char)
  ser.rxflush
  TerminatorChar:=char
  ser.str(STRING("!ST"))
  ser.tx(char)
  RETURN ser.rxtime(500)
                     

PUB AppendStr(str)
  ser.rxflush  
  ser.str(STRING("!AS"))
  ser.str(str)
  RETURN ser.rxtime(5000)
    

PUB ReadStr(str) | char
  ser.rxflush
  char:=TerminatorChar ^ 255 
  ser.str(STRING("!RS"))
  REPEAT UNTIL char == TerminatorChar
    char:=ser.rxtime(500)
    IF char <> TerminatorChar
      BYTE[str]:=char
      str++
  BYTE[str]:=0    
  RETURN ser.rxtime(500)
          

PUB AppendBlock(ptr, count)
  ser.rxflush
  REPEAT UNTIL count < 256
      ser.str(STRING("!AR"))
      ser.tx(255)
      REPEAT 255
        ser.tx(BYTE[ptr])
        ptr++
      count-=255
      ser.rxtime(5000)
  IF count > 0
    ser.rxflush
    ser.str(STRING("!AR"))
    ser.tx(count)
    REPEAT count
      ser.tx(BYTE[ptr])
      ptr++
  RETURN ser.rxtime(5000)
    

PUB ReadBlock(ptr, count)
  ser.rxflush
  REPEAT UNTIL count < 256
    ser.str(STRING("!RR"))
    ser.tx(255)
    REPEAT 255
      BYTE[ptr]:=ser.rxtime(500)
      ptr++
    count-=255
    ser.rxtime(500)
  IF count > 0
    ser.str(STRING("!RR"))
    ser.tx(count)
    REPEAT count
      BYTE[ptr]:=ser.rxtime(500)
      ptr++
  RETURN ser.rxtime(500)

